/*
 * $Id: addvutil.H,v 1.9 1996/08/25 22:54:35 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _ADDVUTIL_H
#define _ADDVUTIL_H

#include <stdio.h>

/***************************************************************************/

int null_strcmp( char *cs, char *ct );
/*
 * Work just linke strcmp().  Treat NULL pointers as < an empty string.
 */

/***************************************************************************/

int null_strlen( char *s );
/*
 * Check to make sure s is non-NULL before a length is returned.
 * If s is NULL, the return value is 0, otherwise the return value
 * is the same as strlen()
 */

/***************************************************************************/

char *null_strdup( char *s );
/*
 * Duplicate a string, allocating space with malloc().
 * if "s" is NULL, return NULL
 */

/***************************************************************************/

char last_char( char *s );
/*
 * Return the last character in the string.
 * Return '\0' if s == NULL
 */

/***************************************************************************/

char *chartostr( char c );
/*
 * Create a newly allocated null-terminated string with c as the first
 * and only character.
 */ 

/***************************************************************************/

char *itoa( int num );
/*
 * Convert the number into a static string.  DO NOT FREE
 */

/***************************************************************************/

char *make_fname( char *path, char *file, char *ext = NULL );
/*
 * Return a newly allocated string containing the concatenation
 * of the fields correctly.  (Add slash and period if necessary.)
 * Do not try to add an extension if NULL.
 */

//---------------------------------------------------------------------------

char *make_fname( char *path, int room, char *ext );
/*
 * Return a newly allocated string containing the path and file of
 * the room with the required extension.  Return NULL if failed.
 */

/***************************************************************************/

char *get_path( char *pathfile );
/*
 * Return a newly allocated string containing just the path
 * portion of a string assumed to consist of a path and file name.
 */

/***************************************************************************/

char *get_file( char *pathfile );
/*
 * Return a newly allocated string containing just the filename and
 * extension portion of a string assumed to consist of a path and file name.
 */

/***************************************************************************/

char *make_url( char *protocol, char *machine, char *path, char *query=NULL );
/*
 * Return a newly allocated string containing a correctly formatted URL.
 * "machine" may be NULL, in which it is correctly omitted.
 */

//---------------------------------------------------------------------------

char *make_url( int room_no, char *query=NULL );
/*
 * A specialized version that returns a newly allocated string that
 * references the room.
 */

/***************************************************************************/

int file_exists( char *path );
/*
 * Returns 1 if the file exists, and 0 if the file does not
 */

/***************************************************************************/

void set_permissions();
/*
 * Set the umask, file permissions, or whatever
 */

/***************************************************************************/

void set_nextroom( char *path, int next_room );
/*
 * Set the value that the next room created will be.
 */

/***************************************************************************/

int get_nextroom( char *path );
/*
 * Get the room number ofr the next room to be created
 */

/***************************************************************************/

typedef enum { BL_NONE, BL_NAME, BL_NUMBER, BL_ENTRY } BacklinkType;

BacklinkType get_backlink_type( char *s );
/*
 * Return a backlink type depending on the string that was sent in.
 * If the string is unparsable, return BL_NONE.
 */

char *get_backlink_string( BacklinkType t, char *roomword );
/*
 * Return a static pointer to a string
 */

/***************************************************************************/

typedef enum { EVIL_NONE, EVIL_ANCHOR, EVIL_COMMENT, EVIL_PRE } EvilHtmlType;

EvilHtmlType has_evil_html( char *s );
/*
 * Indicate what the first kind of HTML formatting that could cause
 * problems is in the string or EVIL_NONE if there is no suspicious
 * formatting.
 */


/***************************************************************************/

#endif

/*
 * $Log: addvutil.H,v $
 * Revision 1.9  1996/08/25 22:54:35  asf
 * Added get_file()
 *
 * Revision 1.8  1996/08/16 04:36:58  asf
 * Added null_strcmp()
 *
 * Revision 1.7  1996/08/04 01:23:24  asf
 * Added chartostr()
 *
 * Revision 1.6  1995/01/28 16:56:58  asf
 * Added support for has_evil_html()
 *
 * Revision 1.5  1995/01/01  16:50:03  asf
 * Added support for the "room word"
 *
 * Revision 1.4  1994/09/16  15:41:48  asf
 * Added null_strdup()
 *
 * Revision 1.3  1994/08/15  19:58:14  asf
 * Added null_strlen() to the public header
 *
 * Revision 1.2  1994/08/08  21:00:20  asf
 * added the backlink functions
 *
 * Revision 1.1  1994/08/04  20:34:13  asf
 * Initial revision
 *
 */
