/*
 * $Id: backlink.H,v 1.6 1996/08/17 00:39:07 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _BACKLINK_H
#define _BACKLINK_H

#include <htmllist.H>

/***************************************************************************/

int backlink_to_room( char *backlink );
/*
 * Extract just the room number from inside the parenthesis.
 * Return 0 if the room number could not be extracted
 */

/***************************************************************************/

class BackLinkNode:public HtmlSNode{
public:
  // Constructor/Destructor
  BackLinkNode(char *title, int sel=0,
               BackLinkNode *prev=NULL, BackLinkNode *next=NULL);
  ~BackLinkNode();

  // Accessors
  char *title();   // Just calls the HtmlNode value()
  int  room();     // Extract just the room number

private:
};

/***************************************************************************/

typedef enum { SORT_TITLE, SORT_ROOM } BacklinkSort;

class BackLink{
public:
  // Constructor/Destructor
  BackLink( char *path, char *roomword );   // Load the file
  ~BackLink();

  // Accessors
  HtmlSList *roomlist();
  HtmlList  *roombullets();

  // Commands
  void sort_name();
  void sort_number();
  /*
   * Sort the list either by room number, or by the full text of
   * the room title.
   */
  
  int  room_exists( int room );
  /*
   * Search for the room number in the list of rooms.  Return 1 if
   * the room exists, 0 if it does not.
   */

  // Operations
  void append_list( char *title );
  void append( char *title );
  /*
   * Append the string both to the end of the list, and to the file.
   */
  
private:
  char *filename;
  HtmlSList *rooms;

  void sort( BacklinkSort kind );
  /*
   * Does the actual sorting
   */
};

/***************************************************************************/

#endif

/*
 * $Log: backlink.H,v $
 * Revision 1.6  1996/08/17 00:39:07  asf
 * Added roombullets()
 *
 * Revision 1.5  1995/01/01 16:42:55  asf
 * Added support for the "room word"
 *
 * Revision 1.4  1994/08/24  18:45:52  asf
 * Added sort() private function and he BacklinkSort type.
 *
 * Revision 1.3  1994/08/14  01:13:59  asf
 * Added append_list()
 *
 * Revision 1.2  1994/08/13  19:15:21  asf
 * Created backlink_to_room()
 * BackLink now accepts a path, not a file, since it knows the fine name.
 *
 * Revision 1.1  1994/08/08  21:00:36  asf
 * Initial revision
 *
 */
