/*
 * $Id: error.C,v 1.15 1996/09/29 13:24:29 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <htmlfile.H>

#include "addvutil.H"
#include "error.H"

/***************************************************************************/

typedef struct{
  AddvErrorType kind;
  char *label;
} AddvErrorLabel;

AddvErrorLabel labellist[] = {
  {ADDVERR_UNKNOWN, "unknown"},
  {ADDVERR_LOCKED, "locked"},
  {ADDVERR_INVALID_TARGET, "invalid_target"},
  {ADDVERR_NOT_EXIST, "not_exist"},
  {ADDVERR_EMPTY, "empty"},
  {ADDVERR_HTML, "html"},
  {ADDVERR_EXISTS, "exists"},
  {ADDVERR_FINISHED, "finished"},
  {ADDVERR_NONEXTROOM, "nonextroom"},
  {ADDVERR_OPT_OVERLOAD, "opt_overload"},
  {ADDVERR_ADDRESS, "address"},
  {ADDVERR_CLOSED, "closed"},
  {ADDVERRLAST, NULL}
};

/***************************************************************************/

void addventure_error( char *filepath, char *module, AddvErrorType kind,
                       char *desc ){
  SetAddv settings( filepath );
  addventure_error( &settings, module, kind, desc );
}

//---------------------------------------------------------------------------
             
void addventure_error( SetAddv *settings, char *module, AddvErrorType kind,
                       char *desc ){

  int backup = 0;    // Should we tell the person to try again?
  HtmlFile f;

  // Find out any header or tailer files to print
  char *headfile = NULL;
  char *tailfile = NULL;
  // Find the base name for the label
  int co = 0;
  while( labellist[co].label && labellist[co].kind != kind ){
    co++;
  }
  char *baselabel = labellist[co].label;
  if( baselabel ){
    char *prefix = "file-error-";

    char *headsuffix = "-head";
    int headlen = strlen(baselabel) + strlen(prefix) + strlen(headsuffix) + 1;
    char *headlabel = (char *)malloc( headlen );
    sprintf( headlabel, "%s%s%s", prefix, baselabel, headsuffix );

    char *tailsuffix = "-tail";
    int taillen = strlen(baselabel) + strlen(prefix) + strlen(tailsuffix) + 1;
    char *taillabel = (char *)malloc( taillen );
    sprintf( taillabel, "%s%s%s", prefix, baselabel, tailsuffix );

    headfile = settings->getPathLabel( headlabel );
    tailfile = settings->getPathLabel( taillabel );
  }

  char *roomw = settings->roomword();
  char *roomword;
  if( roomw ){
    roomword = null_strdup( roomw );
  } else {
    roomword = null_strdup( "room" );
  }

  f.content_type();

  f.title( "Addventure error" );
  f.header( 1, "Addventure error" );

  // Print a header
  if( headfile ){
    f.para();
    f.rawfile( headfile );
    f.para();
  }
  
  switch( kind ){
  case ADDVERR_UNKNOWN:
    f.puts( "An error has occurred. " );
    f.puts( "Your reply was not valid for some reason." );
    backup = 1;
    break;
  case ADDVERR_LOCKED:
    f.puts( "Oops... you seem to have wandered into a " );
    f.puts( roomword );
    f.puts( " still being " );
    f.puts( "built.  Try again in a couple of minutes.\n" );
    backup = 1;
    break;
  case ADDVERR_INVALID_TARGET:
    f.puts( "One or more of the targets you requested is not a legal target ");
    f.puts( roomword );
    f.puts( "."  );
    backup = 1;
    break;
  case ADDVERR_NOT_EXIST:
    f.puts( "The " );
    f.puts( roomword );
    f.puts( " you are trying to edit was not created." );
    break;
  case ADDVERR_EMPTY:
    f.puts( "You either did not enter a description, or tried to create a " );
    f.puts( roomword );
    f.puts( " with zero options." );
    backup = 1;
    break;
  case ADDVERR_HTML:
    char *reason[4];
    f.puts( "The following HTML codes are not allowed:" );
    reason[0] = "Anchors, &lt;a&gt; and &lt;/a&gt;, in option descriptions.";
    reason[1] = "Preformatting, &lt;pre&gt; and &lt;/pre&gt;, in either"
      "the text or option descriptions.";
    reason[2] = "Comments in either the text or option descriptions.";
    reason[3] = NULL;
    f.unnumlist( reason );
    backup = 1;
    break;
  case ADDVERR_EXISTS:
    f.puts( "Somehow, that " );
    f.puts( roomword );
    f.puts( " was already being created." );
    break;
  case ADDVERR_FINISHED:
    f.puts( "The " );
    f.puts( roomword );
    f.puts( " has already been created.  It is too late to remove it." );
    break;
  case ADDVERR_NONEXTROOM:
    f.puts( "For some reason, Addventure could not allocate a new " );
    f.puts( roomword );
    f.puts( " for your choices.  Your " );
    f.puts( roomword );
    f.puts( " was not created." );
    break;
  case ADDVERR_OPT_OVERLOAD:
    f.puts( "Sorry, you are not allowed to create that many episodes." );
    backup = 1;
    break;
  case ADDVERR_ADDRESS:
    f.puts( "That does not appear to be a valid email address.  " );
    f.puts( "Please enter a valid address, or do not request an Addvanced ");
    f.puts( "option that requires an email address." );
    backup = 1;
    break;
  case ADDVERR_CLOSED:
    f.puts( "You can not expand to this game at the moment." );
    break;
  }
  f.para();

  if( backup ){
    f.puts( "Please go back a screen and try again.\n" );
  }

  if( desc ){
    f.raw( desc );
    f.para();
  }

  // Print a footer
  if( tailfile ){
    f.para();
    f.rawfile( tailfile );
    f.para();
  }
  
  f.line();
  f.puts( "Problem occurred in: <i>" );
  f.puts( module );
  f.puts( "</i>" );
  f.para();

  free( roomword );
}

/***************************************************************************/

/*
 * $Log: error.C,v $
 * Revision 1.15  1996/09/29 13:24:29  asf
 * Changed text for ADVERR_ADDRESS
 *
 * Revision 1.14  1996/07/10 13:09:28  asf
 * Fixed problem with failure to dereference the settings variable.
 *
 * Revision 1.13  1996/05/02 21:06:10  asf
 * Many changes to support the headfile and tailfile.
 * Added the text for the Lockfile Deleted.
 *
 * Revision 1.12  1996/03/24  17:43:53  asf
 * Added ADDVERR_CLOSED
 *
 * Revision 1.11  1996/03/24  13:48:53  asf
 * Added ADDVERR_ADDRESS
 *
 * Revision 1.10  1996/03/22  22:28:49  asf
 * Added the case for ADVERR_OPT_OVERLOAD
 *
 * Revision 1.9  1995/10/05  21:09:24  asf
 * Added the No Next Room error
 *
 * Revision 1.8  1995/05/26  16:02:22  asf
 * Fixed a compile problem with addventure_error().
 *
 * Revision 1.7  1995/01/28  17:00:25  asf
 * Added more types of bad HTML that is reported.
 *
 * Revision 1.6  1995/01/01  23:01:05  asf
 * Fixed some spacing.
 *
 * Revision 1.5  1995/01/01  17:37:36  asf
 * Added the "room word" to the error messages
 *
 * Revision 1.4  1994/12/25  11:07:38  asf
 * Added ADDVERR_HTML
 *
 * Revision 1.3  1994/12/17  19:11:25  asf
 * Changed the message for ADDVERR_EMPTY slightly (which really makes the name
 * wrong.. but...)
 *
 * Revision 1.2  1994/09/12  12:00:50  asf
 * Moved the line that reported the module to a less obvious place.
 *
 * Revision 1.1  1994/08/26  09:08:52  asf
 * Initial revision
 *
 */
