/*
 * $Id: makeroom.C,v 1.9 1996/08/15 23:19:04 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>

#include <htmltext.H>
#include <htmlfile.H>

#include "version.H"
#include "makeroom.H"
#include "addvutil.H"
#include "setaddv.H"

/***************************************************************************/

int  make_room( HtmlFile *output, 
                char *filepath, char *docpath, char *cgipath,
                char *roomword, int room_no, int parent_room,
                int allow_java ){
  /*
   * Create the first page seen when a new room must be created.
   * The room should contain the room number and parent room number,
   * and needs to send this information to "newproc".
   */
  /*
   * Output info saying the page hasn't been created, and they can
   * go create it now.  Does not do any checking to make sure this
   * is a very intelligent thing to do.
   */

  char *pathbin;      // The full path and name of the CGI scrpt
  char *pathbinpath;  // Above, plus the path (relative to ServerHome)
                      //   of the room files
  char *newproc;      // The newproc CGI script URL

  char temp[80];      // A temporary string

  SetAddv setting( filepath );
  output->debug( output->get_debug() || setting.debug( "makeroom-output" ) );
  
  // Create the title and header
  sprintf( temp, "Creating %s %d", roomword, room_no );
  output->title( temp );
  output->header( 1, temp );

  // Create standard text
  output->puts( "This " );
  output->puts( roomword );
  output->puts( " has not yet been created.  " );
  output->puts( "If you wish, you may either create it now or not create it.\n" );

  // Create the form contents
  HtmlText f;

  char *parent_url = make_url( parent_room );

  f.link( parent_url, "Cancel" );
  f.puts( " - Do <b>not</b> create the " );
  f.puts( roomword );
  f.puts( "." );  f.para();
  f.submit( "Accept" );
  f.puts( " - Create the " );
  f.puts( roomword );
  f.puts( "!" );  f.para();

  f.line();
  f.puts( "Ignore anything below this line.\n" );
  f.hidden( "room", itoa( room_no ) );    f.para();
  f.hidden( "parent", itoa( parent_room ) );  f.para();
  f.hidden( "version", VERSION_NUM );  f.para();
  if( !allow_java ){
    // Don't include java
    f.hidden( "nojava", "true" );  f.para();
  }

  // Compute what to execute and put the form into the document
  pathbin = make_fname( cgipath, NEWPROC );
  pathbinpath = make_fname( pathbin, docpath );
  newproc = make_url( "http", NULL, pathbinpath );
  output->form( newproc, METHOD_POST, f.val() );
  free( pathbinpath );
  free( pathbin );
  free( newproc );

  return 1;   // @@@
}

//---------------------------------------------------------------------------

int make_room( char *filepath, char *docpath, char *cgipath,
               char *roomword, int room_no, int parent_room ){
  /*
   * Create a page saying the page hasn't been created, and they can
   * go create it now.  The resultant creation will use java if
   * installed.
   * Return 1 if successfull, return 0 if not.
   */

  int  ret;        // return value
  char *file;      // The name of the file
  FILE *room;      // Output the room to a file

  SetAddv setting( filepath );

  // Determine the file name
  file = make_fname( filepath, room_no, "html" );
  if(setting.debug("makeroom-filename"))
    fprintf( stderr, "makeroom filename='%s'\n", file );

  if( file_exists( file ) ){
    // Error!  We can't make a room if it already exists.

    ret = 0;
  } else {

    if( room = fopen( file, "w" ) ){
      // File was opened.  Output the text and form

      HtmlFile r( room );  // The object that stores the entire room

      r.debug( setting.debug("makeroom-file") );

      // Create it
      ret = make_room( &r, filepath, docpath, cgipath,
                       roomword, room_no, parent_room );

      // Close the file
      fclose( room );
    } else {
      // File not opened.
      ret = 0;  // Failed
    }

  } // If/else file exists
  
  return ret;
}

/***************************************************************************/

/*
 * $Log: makeroom.C,v $
 * Revision 1.9  1996/08/15 23:19:04  asf
 * make_room() still creates a page telling the user they can create the
 * room or back out.  It still does this, but now there is a make_room()
 * that prevents the creation page from using java, and the new make_room()
 * can send the HTML to any HtmlText object passed to it.
 * The old make_room() actually just calls the new make_room().
 * Added debugging.
 *
 * Revision 1.8  1996/05/03 07:28:36  asf
 * Save the version number we were created with.  This may be used
 * in the future.
 *
 * Revision 1.7  1995/10/05  21:10:13  asf
 * Added error checking to make sure that the room being made doesn't
 * already exist.
 *
 * Revision 1.6  1995/01/01  18:27:38  asf
 * Fixed some spacing problems.
 *
 * Revision 1.5  1995/01/01  16:43:50  asf
 * Added support for the "room word"
 *
 * Revision 1.4  1994/10/01  15:53:40  asf
 * Removed make_lock() (Functionality transfered to LockFile object.)
 *
 * Revision 1.3  1994/08/25  01:30:36  asf
 * Made the buttons clearer.
 *
 * Revision 1.2  1994/08/14  14:33:42  asf
 * Added CGI-script name definition
 *
 * Revision 1.1  1994/08/04  20:34:37  asf
 * Initial revision
 *
 */
