/*
 * $Id: setting.C,v 1.5 1996/08/17 20:19:19 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1991-1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "setting.H"

extern "C"{
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
}

#include "addvutil.H"

#ifndef remove
#define remove( x )  unlink( x )
#endif

/* transition [current_state] [character_class] = next_state */
short transition[8][5] = { { 0, 0, 0, 0, 0 },
                           { 0, 1, 2, 3, 0 },
                           { 0, 0, 2, 3, 0 },
                           { 0, 4, 7, 0, 0 },
                           { 0, 4, 6, 0, 5 },
                           { 0, 1, 1, 1, 1 },
                           { 0, 4, 6, 0, 5 },
                           { 0, 4, 7, 0, 0 } };

/***************************************************************************/

short character_class( char c ){
/*
 * Returns the character class for the character passed to it.
 *     CLASS          CHARACTERS
 *       1              A-Z, a-z, 0-9, -, _, +, /, $
 *       2              space
 *       3              =
 *       4              \0
 *       0              all other characters
 */
   short ret;

   if( c == ' ' ) {
      ret = 2;
   } else if( c == '=' ) {
      ret = 3;
   } else if( (c == '\0') || (c == '\n') ) {
      ret = 4;
   } else if( (isalnum(c) || ispunct(c)) && (c != '=') ){
      ret = 1;
   } else ret = 0;
   return( ret );
} /* end character_class */

/***************************************************************************/

int process_line( char *inln, char *label, char *value ){
/*
 * Processes a line that is passed by inln.  Either places a label name
 * in the space pointed to by label and the value of that label in value,
 * or returns the comment (including # sign) in inln.  The other fields
 * are cleared out.  Returns a 1 if successfull or 0 if error.  Line with
 * error is returned in inln.  Label, and value should be pointers to
 * allocated memory (usually from arrays of characters).
 */
  int co;
  short state = 1;
  int ret = 1;
  int numspc = 0; 
  char outstring[80];
  int outpos;
  int inpos;
  short cclass;

  if( (inln[0] == '#') ){
    /* comment */
    label[0] = value[0] = '\0';
  } else if( isspace( inln[0] ) ){
    /* leading space */
    label[0] = value[0] = '\0';
  } else {
    /* scan the line */
    outstring[0] = '\0';
    outpos = 0;
    inpos = 0;
    while( (state != 0) && (state != 5) ){

      /* What state shoudl we be in? */
      cclass = character_class( inln[inpos] );
      state = transition[state][cclass];

      /* Add spaces if appropriate */
      if( (state == 4) && (numspc) ){
        for( co=1; co<=numspc; co++ ){
          outstring[outpos++] = ' ';
          outstring[outpos] = '\0';
        }
        numspc = 0;
      }

      /* Add the normal characters */
      if( cclass == 1 ) {
        outstring[outpos++] = inln[inpos];
        outstring[outpos] = '\0';
      }

      /* Take actions depending on our state */
      if( state == 0 ){
        ret = 0;
        label[0] = value[0] = '\0';
      } else if( state == 3 ){
        strcpy( label, outstring );
        outstring[ (outpos=0) ] = '\0';
      } else if( state == 5 ){
        strcpy( value, outstring );
        inln[0] = '\0';
      } else if( state == 6 ){
        numspc++;
      }

      /* Advance one */
      inpos++;
    } /* end while */
  } /* end if/else */
  return( ret );
} /* process_line */

/***************************************************************************/

Setting::Setting( char *path )
{
  int l;             // The length of the string to allocate
  char slash[2];     // We may or may not need a slash

  const char *name = "options.addv";
  
  if( last_char( path ) == '/' ){
    // The slash is already there
    strcpy( slash, "" );
  } else {
    // We will need a slash
    strcpy( slash, "/" );
  }

  l = null_strlen( path ) + null_strlen( slash ) + null_strlen( name ) + 1;
  filename = (char *)malloc( l );
  sprintf( filename, "%s%s%s", path, slash, name );
}

/***************************************************************************/

Setting::~Setting()
{
  free( filename );
}

/***************************************************************************/

char *Setting::getLabel( char *label )
{
  /*
   * Reads the file for the case sensitive label.
   * If found, allocates memory for the value and returns a pointer to it.
   * If not found, NULL is returned.
   */
  
  FILE *infile;
  char inln[80];
  char inlabel[80];
  char invalue[80];
  char *ret = NULL;
  int ok = 1;
  int done = 0;

  if( infile=fopen( filename, "r" ) ){
    /* file exists */
    while( !done ){
      if( fgets( inln, 80, infile ) != NULL ){
        ok = process_line( inln, inlabel, invalue );
        if( !strcmp( inlabel, label ) ){
          done = 1;
          ret = (char *)malloc( null_strlen( invalue )+1 );
          if( ret != NULL ) strcpy( ret, invalue );
        } /* end if */
      } else done = 1;
    } /* end while */
  } /* end if */
  fclose( infile );

  return( ret );
}

/*-------------------------------------------------------------------------*/

char *Setting::getLabel( char *label, char *def )
{
  char *ret;

  ret = getLabel( label );
  if( !ret ){
    ret = def;
  }

  return ret;
}

/***************************************************************************/

int Setting::setLabel( char *label, char *value )
{
  /*
   * Reads the file and searches for the line
   * containing the label.  Other lines are written out to a temporary
   * file.  If the label is found, it and the value are written to the file
   * in the form "label = value".  It will only change the first found
   * matching label.  If the label is not found, it and the value will be
   * written at the end of the file.  The old file is then deleted and
   * the temp file is copied to it.
   * 
   * Returns 0 if an error was detected (some of the operation may have
   * succeeded).  Returns 1 if success.
   */
  char *tempfn;
  FILE *infile;
  FILE *outfile;
  char inln[80];
  char inlabel[80];
  char invalue[80];
  int ok = 1;
  int done = 0;
  int inv_ret = 0;  /* The inverse of the return value */

  /*
   * It is easier to turn something on to indicate an error, so inv_ret is
   * set to 1 (or the return value of fclose()) if there is an error.  This
   * has to be inverted before we return it so that an error is indicated by
   * a 0.  This is consistent with p_get_value and friends.
   */

  /* Get a name for the temporary file */
  tempfn = tmpnam(NULL);
  
  if( infile=fopen( filename, "r" ) ){
    /* file exists */
    outfile = fopen( tempfn, "w" );
    while( !done ){
      if( fgets( inln, 80, infile ) != NULL ){
        ok = process_line( inln, inlabel, invalue );
        if( (!strcmp( inlabel, label )) && (ok) ){
          /* The labels match */
          done = 2;
          if( value != NULL ){
            // If we have a new value, store it
            fprintf( outfile, "%s = %s\n", label, value );
          } // Otherwise this never gets written out
        } else {
          if( ok ){
            if( inln[0] ){
              /* a comment */
              fputs( inln, outfile );
            } else {
              fprintf( outfile, "%s = %s\n", inlabel, invalue );
            } /* end if */
          } else {
            /* error reading the file */
            fprintf( stderr, "\007Error reading data: %s\n", inln );
            inv_ret = 1;        /* set the error return code */
          } /* end if */
        } /* end if */
      } else done = 1;
    } /* end while */
  } /* end if */
  if( done == 2 ){
    /* we matched, copy the rest of the file */
    while( fgets( inln, 80, infile ) != NULL ){
      fputs( inln, outfile );
    } /* end while */
  } else if( done == 1) {
    /* no match, stick it at the end */
    fprintf( outfile, "%s = %s\n", label, value );
  } /* end if */
  if( done != 0 ){
    inv_ret = inv_ret || fclose( outfile );
    inv_ret = inv_ret || fclose( infile );
    /*
     * Delete the old file and copy the temporary file to the permanent
     * one.  Then delete the temp file.
     */
    remove( filename );
    infile = fopen( tempfn, "r" );
    outfile = fopen( filename, "w" );
    while( fgets( inln, 80, infile ) != NULL ){
      fputs( inln, outfile );
    } /* end while */
    inv_ret = inv_ret || fclose( outfile );
    inv_ret = inv_ret || fclose( infile );
    remove( tempfn );
  } else {
    /* The file did not already exist */
    outfile = fopen( filename, "w" );
    fprintf( outfile, "%s = %s\n", label, value );
    inv_ret = inv_ret || fclose( outfile );
  } /* end if */

  return( !inv_ret ); /* return ZERO if error */
  
}

/***************************************************************************/

/*
 * $Log: setting.C,v $
 * Revision 1.5  1996/08/17 20:19:19  asf
 * Changed copyright
 * Request to set a NULL value causes the label to be cleared.
 *
 * Revision 1.4  1994/10/08 10:07:35  asf
 * Allowerd spaced in the value part of a setting.
 *
 * Revision 1.3  1994/09/14  13:48:06  asf
 * Removed the set() function.  (Now in SetAddv)
 *
 */
