/*
 * $Id: undoroom.C,v 1.11 1996/08/15 23:11:40 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Allow a person to fix-up a room that they didn't finish.
 */

#include <htmlfile.H>

#include <unistd.h>

#include "undo.H"
#include "cleanup.H"
#include "addvutil.H"
#include "makeroom.H"
#include "openfile.H"
#include "error.H"
#include "setaddv.H"

/***************************************************************************/

void entry_nojava( char *filepath, char *urlpath, char *binpath,
                   char *roomword, int room, int parent ){

  // Send something out to the user allowing them to create a page
  // without java stuff.

  // Start the page
  HtmlFile f(stdout);
  f.content_type();

  SetAddv setting( filepath );
  f.debug(setting.debug("undoroom-return") ||
          setting.debug("undoroom-nojava") );

  make_room( &f, filepath, urlpath, binpath, roomword, room, parent, 0 );
}

//---------------------------------------------------------------------------

void entry( char *filepath, char *urlpath,
            char *roomword, int room, int parent ){
  char pagetitle[160];
  char *file;
  char *url;

  // Start the page
  HtmlFile f(stdout);

  SetAddv setting( filepath );
  f.debug(setting.debug("undoroom-return") ||
          setting.debug("undoroom-entry") );
  
  f.content_type();

  // Generate the title
  sprintf( pagetitle, "%s %d cleaned", roomword, room );
  f.title( pagetitle );

  // Generate the header
  f.header( 1, pagetitle );

  f.puts( "The " );
  f.puts( roomword );
  f.puts( " has been cleaned and is ready to be created again.\n" );

  // Create the link to the room
  file = make_fname( urlpath, room, "html" );
  url = make_url( NULL, NULL, file, "new" );   // Need to append a fake query
  f.link( url, "Enter!" );
  f.puts( " - Go to the " );
  f.puts( roomword );
  f.puts( " and try again\n" );
  free( url );
  free( file );

  // Create the link to the parent, just in case
  file = make_fname( urlpath, parent, "html" );
  url = make_url( NULL, NULL, file );
  f.link( url, "Go back" );
  f.puts( " - Go to the parent " );
  f.puts( roomword );
  f.puts( ".\n" );
  free( url );
  free( file );
}

/***************************************************************************/

main( int argc, char *argv[] ){
  char *indexstr;   // The index/query value
  int  room;
  int  parent;

  char *urlpath;   // The path relative to the server root
  char *filepath;  // The absolute path in the file system
  char *binpath;   // Where the cgi scrpts are

  if( argc > 1 ){
    // There is a first value

    // Get the data passed by environment variables
    urlpath  = getenv( "PATH_INFO" );
    filepath = getenv( "PATH_TRANSLATED" );
    binpath  = get_path( getenv( "SCRIPT_NAME" ) );

    SetAddv setting( filepath );
    char *roomword = setting.roomword();

    // Read the value and break it into room and parent
    indexstr = argv[1];
    Undo undo( indexstr );
    room = undo.room();
    parent = undo.parent();

    // @@@ Should check to make sure the right person is doing this

    OpenFile openfile( filepath, room );
    if( !openfile.exists() ){
      // They're trying to undo a finished room!
      addventure_error( &setting, UNDOROOM, ADDVERR_FINISHED );
    } else {
      // Clean it up
      cleanup( room, parent, roomword, filepath, urlpath, binpath );
      
      // Return something
      if( undo.dojava() ){
        // We can try to re-enter, and allow applets to be used
        entry( filepath, urlpath, roomword, room, parent );
      } else {
        // No java, please, so bring up a special "new room" page
        entry_nojava( filepath, urlpath, binpath, roomword, room, parent );
      }
    } 
  } else {
    //Only the title was present.  No query value.  Oops.
  }
}

/***************************************************************************/

/*
 * $Log: undoroom.C,v $
 * Revision 1.11  1996/08/15 23:11:40  asf
 * Changes to allow the undoroom program to return a page allowing the user
 * to create page without using any java applet tags.
 * Added debugging and some changes to support.
 *
 * Revision 1.10  1996/05/02 21:11:49  asf
 * Changes to support the addventure_error() changes.
 *
 * Revision 1.9  1995/02/12  12:39:55  asf
 * Changes to "correctly" support relative addressing.
 *
 * Revision 1.8  1995/01/01  21:58:04  asf
 * Fixed a parameter typo in the call to cleanup()
 * Added "room word" support for addventure_error
 *
 * Revision 1.7  1995/01/01  16:31:23  asf
 * Added support for the "room word"
 *
 * Revision 1.6  1994/10/01  15:52:47  asf
 * Switched to use of LockFile and OpenFile.
 *
 * Revision 1.5  1994/09/25  11:16:27  asf
 * Removed code that has been moved to the cleanup module
 *
 * Revision 1.4  1994/08/26  09:08:20  asf
 * Converted to error package
 *
 * Revision 1.3  1994/08/25  01:08:32  asf
 * Added openfile support.
 *
 * Revision 1.2  1994/08/23  20:47:55  asf
 * Actually does a remove()!  Radical.  Idiot.
 *
 * Revision 1.1  1994/08/20  22:06:35  asf
 * Initial revision
 *
 */






